@@ -1,7 +1,7 @@
 package DBD::SalesForce::st;
 
 # ----------------------------------------------------------------------
-# $Id: st.pm,v 1.2 2005/11/14 12:58:06 shimizu Exp $
+# $Id: st.pm,v 1.4 2005/11/21 12:06:16 shimizu Exp $
 # ----------------------------------------------------------------------
 # DBD::SalesForce::st - Statement handle
 # ----------------------------------------------------------------------
@@ -11,7 +11,7 @@ use base qw(DBD::_::st);
 use vars qw($VERSION $imp_data_size);
 
 use DBI;
-#use Data::Dumper;
+use Data::Dumper;
 
 $VERSION = "0.01";
 $imp_data_size = 0;
@@ -32,9 +32,8 @@ sub execute {
     $parser    = $sth->{'Parser'};
     $statement = $sth->{'Statement'};
 
-#print Dumper $parser;
-
     if ($parser->{'command'} eq "SELECT") {
+    
         # Create valid SQL for Salesforce::query
         my $sql = sprintf("SELECT %s FROM %s", 
             join(",", @{$parser->{'org_col_names'}}), $parser->{'table_names'}->[0]);
@@ -43,7 +42,7 @@ sub execute {
         # The Salesforce::query instance
         $search = $salesforce->query(query => $sql, limit => $limit);
         if ($search->valueof('//Body/Fault')) {
-            my $errstr = $search->valueof('//Body/Fault/detail/fault')->{'exceptionCode'} . 
+            my $errstr = $search->valueof('//Body/Fault/detail/fault')->{'exceptionCode'} . "\n" . 
                          $search->valueof('//Body/Fault/detail/fault')->{'exceptionMessage'};
             $sth->{'Database'}->set_err(1, $errstr);
         }
@@ -66,6 +65,53 @@ sub execute {
             }
             push @data, \@this;
         }
+        
+    } elsif ($parser->{'command'} eq "UPDATE") {
+    
+        my %in;
+        $in{'Id'} = $parser->{'where_cols'}->{'Id'}->[0];
+        $in{'type'} = $parser->{'table_names'}->[0];
+        for (my $i = 0; $i < scalar(@{$parser->{'column_names'}}); $i++) {
+            $in{$parser->{'column_names'}->[$i]} = $parser->{'values'}->[$i]->{'value'};
+        }
+
+        # The Salesforce::update instance
+        $search = $salesforce->update(%in);
+        if ($search->valueof('//Body/updateResponse/result/errors')) {
+            my $errstr = $search->valueof('//Body/updateResponse/result/errors')->{'statusCode'} . "\n" .
+                         $search->valueof('//Body/updateResponse/result/errors')->{'message'};
+            $sth->{'Database'}->set_err(1, $errstr);
+        }
+
+    } elsif ($parser->{'command'} eq "INSERT") {
+    
+        my %in;
+        $in{'type'} = $parser->{'table_names'}->[0];
+        for (my $i = 0; $i < scalar(@{$parser->{'column_names'}}); $i++) {
+            $in{$parser->{'column_names'}->[$i]} = $parser->{'values'}->[$i]->{'value'};
+        }
+
+        # The Salesforce::create instance
+        $search = $salesforce->create(%in);
+        if ($search->valueof('//Body/Fault')) {
+            my $errstr = $search->valueof('//Body/Fault/detail/fault')->{'exceptionCode'} . "\n" . 
+                         $search->valueof('//Body/Fault/detail/fault')->{'exceptionMessage'};
+            $sth->{'Database'}->set_err(1, $errstr);
+        }
+
+    } elsif ($parser->{'command'} eq "DELETE") {
+    
+        my %in;
+        $in{'Id'} = $parser->{'where_cols'}->{'Id'}->[0];
+        
+        # The Salesforce::delete instance
+        $search = $salesforce->delete(%in);
+        print Dumper $search->valueof('//Body/deleteResponse/result');
+        if ($search->valueof('//Body/deleteResponse/result/errors')) {
+            my $errstr = $search->valueof('//Body/deleteResponse/result/errors')->{'statusCode'} . "\n" .
+                         $search->valueof('//Body/deleteResponse/result/errors')->{'message'};
+            $sth->{'Database'}->set_err(1, $errstr);
+        }
     }
 
     $sth->{'driver_data'} = \@data;
